@@ -62,53 +62,6 @@ static int xattr_tdb_get_file_id(struct vfs_handle_struct *handle,
 	return 0;
 }
 
-static ssize_t xattr_tdb_getxattr(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				const char *name,
-				void *value,
-				size_t size)
-{
-	struct file_id id;
-	struct db_context *db;
-	ssize_t xattr_size;
-	int ret;
-	DATA_BLOB blob;
-	TALLOC_CTX *frame = talloc_stackframe();
-
-	SMB_VFS_HANDLE_GET_DATA(handle, db, struct db_context,
-				if (!xattr_tdb_init(-1, frame, &db))
-				{
-					TALLOC_FREE(frame); return -1;
-				});
-
-	ret = xattr_tdb_get_file_id(handle, smb_fname->base_name, &id);
-	if (ret == -1) {
-		TALLOC_FREE(frame);
-		return -1;
-	}
-
-	xattr_size = xattr_tdb_getattr(db, frame, &id, name, &blob);
-	if (xattr_size < 0) {
-		errno = ENOATTR;
-		TALLOC_FREE(frame);
-		return -1;
-	}
-
-	if (size == 0) {
-		TALLOC_FREE(frame);
-		return xattr_size;
-	}
-
-	if (blob.length > size) {
-		TALLOC_FREE(frame);
-		errno = ERANGE;
-		return -1;
-	}
-	memcpy(value, blob.data, xattr_size);
-	TALLOC_FREE(frame);
-	return xattr_size;
-}
-
 struct xattr_tdb_getxattrat_state {
 	struct vfs_aio_state vfs_aio_state;
 	ssize_t xattr_size;
@@ -283,35 +236,6 @@ static ssize_t xattr_tdb_fgetxattr(struct vfs_handle_struct *handle,
 	return xattr_size;
 }
 
-static int xattr_tdb_setxattr(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				const char *name,
-				const void *value,
-				size_t size,
-				int flags)
-{
-	struct file_id id;
-	struct db_context *db;
-	int ret;
-	TALLOC_CTX *frame = talloc_stackframe();
-
-	SMB_VFS_HANDLE_GET_DATA(handle, db, struct db_context,
-				if (!xattr_tdb_init(-1, frame, &db))
-				{
-					TALLOC_FREE(frame); return -1;
-				});
-
-	ret = xattr_tdb_get_file_id(handle, smb_fname->base_name, &id);
-	if (ret == -1) {
-		TALLOC_FREE(frame);
-		return -1;
-	}
-
-	ret = xattr_tdb_setattr(db, &id, name, value, size, flags);
-	TALLOC_FREE(frame);
-	return ret;
-}
-
 static int xattr_tdb_fsetxattr(struct vfs_handle_struct *handle,
 			       struct files_struct *fsp,
 			       const char *name, const void *value,
@@ -342,34 +266,6 @@ static int xattr_tdb_fsetxattr(struct vfs_handle_struct *handle,
 
 }
 
-static ssize_t xattr_tdb_listxattr(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				char *list,
-				size_t size)
-{
-	struct file_id id;
-	struct db_context *db;
-	int ret;
-	TALLOC_CTX *frame = talloc_stackframe();
-
-	SMB_VFS_HANDLE_GET_DATA(handle, db, struct db_context,
-				if (!xattr_tdb_init(-1, frame, &db))
-				{
-					TALLOC_FREE(frame); return -1;
-				});
-
-	ret = xattr_tdb_get_file_id(handle, smb_fname->base_name, &id);
-	if (ret == -1) {
-		TALLOC_FREE(frame);
-		return -1;
-	}
-
-	ret = xattr_tdb_listattr(db, &id, list, size);
-	TALLOC_FREE(frame);
-	return ret;
-
-}
-
 static ssize_t xattr_tdb_flistxattr(struct vfs_handle_struct *handle,
 				    struct files_struct *fsp, char *list,
 				    size_t size)
@@ -398,33 +294,6 @@ static ssize_t xattr_tdb_flistxattr(struct vfs_handle_struct *handle,
 	return ret;
 }
 
-static int xattr_tdb_removexattr(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				const char *name)
-{
-	struct file_id id;
-	struct db_context *db;
-	int ret;
-	TALLOC_CTX *frame = talloc_stackframe();
-
-	SMB_VFS_HANDLE_GET_DATA(handle, db, struct db_context,
-				if (!xattr_tdb_init(-1, frame, &db))
-				{
-					TALLOC_FREE(frame); return -1;
-				});
-
-	ret = xattr_tdb_get_file_id(handle, smb_fname->base_name, &id);
-	if (ret == -1) {
-		TALLOC_FREE(frame);
-		return ret;
-	}
-
-	
-	ret = xattr_tdb_removeattr(db, &id, name);
-	TALLOC_FREE(frame);
-	return ret;
-}
-
 static int xattr_tdb_fremovexattr(struct vfs_handle_struct *handle,
 				  struct files_struct *fsp, const char *name)
 {
@@ -616,6 +485,7 @@ static int xattr_tdb_unlinkat(vfs_handle_struct *handle,
 			int flags)
 {
 	struct smb_filename *smb_fname_tmp = NULL;
+	struct smb_filename *full_fname = NULL;
 	struct file_id id;
 	struct db_context *db;
 	int ret = -1;
@@ -635,24 +505,26 @@ static int xattr_tdb_unlinkat(vfs_handle_struct *handle,
 		return -1;
 	}
 
-	if (smb_fname_tmp->flags & SMB_FILENAME_POSIX_PATH) {
-		ret = SMB_VFS_NEXT_LSTAT(handle, smb_fname_tmp);
+	/*
+	 * TODO: use SMB_VFS_STATX() once we have that
+	 */
+
+	full_fname = full_path_from_dirfsp_atname(frame,
+						  dirfsp,
+						  smb_fname);
+	if (full_fname == NULL) {
+		goto out;
+	}
+
+	if (full_fname->flags & SMB_FILENAME_POSIX_PATH) {
+		ret = SMB_VFS_NEXT_LSTAT(handle, full_fname);
 	} else {
-		ret = SMB_VFS_NEXT_STAT(handle, smb_fname_tmp);
-		if (ret == -1 && (errno == ENOENT || errno == ELOOP)) {
-			if (VALID_STAT(smb_fname->st) &&
-					S_ISLNK(smb_fname->st.st_ex_mode)) {
-				/*
-				 * Original name was a link - Could be
-				 * trying to remove a dangling symlink.
-				 */
-				ret = SMB_VFS_NEXT_LSTAT(handle, smb_fname_tmp);
-			}
-		}
+		ret = SMB_VFS_NEXT_STAT(handle, full_fname);
 	}
 	if (ret == -1) {
 		goto out;
 	}
+	smb_fname_tmp->st = full_fname->st;
 
 	if (flags & AT_REMOVEDIR) {
 		/* Always remove record when removing a directory succeeds. */
@@ -731,15 +603,11 @@ static int xattr_tdb_connect(vfs_handle_struct *handle, const char *service,
 }
 
 static struct vfs_fn_pointers vfs_xattr_tdb_fns = {
-	.getxattr_fn = xattr_tdb_getxattr,
 	.getxattrat_send_fn = xattr_tdb_getxattrat_send,
 	.getxattrat_recv_fn = xattr_tdb_getxattrat_recv,
 	.fgetxattr_fn = xattr_tdb_fgetxattr,
-	.setxattr_fn = xattr_tdb_setxattr,
 	.fsetxattr_fn = xattr_tdb_fsetxattr,
-	.listxattr_fn = xattr_tdb_listxattr,
 	.flistxattr_fn = xattr_tdb_flistxattr,
-	.removexattr_fn = xattr_tdb_removexattr,
 	.fremovexattr_fn = xattr_tdb_fremovexattr,
 	.openat_fn = xattr_tdb_openat,
 	.mkdirat_fn = xattr_tdb_mkdirat,
