@@ -27,7 +27,6 @@
 #  PYTHONPATH="$PYTHONPATH:$samba4srcdir/torture/drs/python" $SUBUNITRUN replica_sync -U"$DOMAIN/$DC_USERNAME"%"$DC_PASSWORD"
 #
 
-from __future__ import print_function
 import drs_base
 import samba.tests
 import time
@@ -95,7 +94,7 @@ class DrsReplicaSyncTestCase(drs_base.DrsBaseTestCase):
 
         # Tunnel the command line credentials down to the
         # subcommand to avoid a new kinit
-        cmdline_auth = "--krb5-ccache=%s" % ccache_name
+        cmdline_auth = "--use-krb5-ccache=%s" % ccache_name
 
         # bin/samba-tool drs <drs_command> <cmdline_auth>
         cmd_list = ["drs", "replicate", cmdline_auth]
